@@ -42,6 +42,7 @@ static int handle_cmdline(int *argc, char ***argv);
 static void close_fds(void);
 static int daemonize(void);
 static int open_log(void);
+static int create_pidfile(void);
 static void clean_exit(int sig);
 static void reload_conf(int sig);
 static char *read_line(int fd);
@@ -49,14 +50,19 @@ static char *read_line(int fd);
 /* global debug level */
 int acpid_debug;
 
+/* do we log event info? */
+int logevents;
+
 static const char *progname;
-static const char *confdir = ACPI_CONFDIR;
-static const char *eventfile = ACPI_EVENTFILE;
-static const char *socketfile = ACPI_SOCKETFILE;
+static const char *confdir = ACPID_CONFDIR;
+static const char *eventfile = ACPID_EVENTFILE;
+static const char *socketfile = ACPID_SOCKETFILE;
+static const char *lockfile = ACPID_LOCKFILE;
 static int nosocket;
 static const char *socketgroup;
-static mode_t socketmode = ACPI_SOCKETMODE;
+static mode_t socketmode = ACPID_SOCKETMODE;
 static int foreground;
+static const char *pidfile = ACPID_PIDFILE;
 
 int
 main(int argc, char **argv)
@@ -168,9 +174,18 @@ main(int argc, char **argv)
 	signal(SIGPIPE, SIG_IGN);
 
 	/* read in our configuration */
-	acpid_read_conf(confdir);
+	if (acpid_read_conf(confdir)) {
+		exit(EXIT_FAILURE);
+	}
+
+	/* create our pidfile */
+	if (create_pidfile() < 0) {
+		exit(EXIT_FAILURE);
+	}
 
 	/* main loop */
+	acpid_log(LOG_INFO, "waiting for events: event logging is %s\n",
+	    logevents ? "on" : "off");
 	while (1) {
 		struct pollfd ar[2];
 		int r;
@@ -193,7 +208,12 @@ main(int argc, char **argv)
 		/* was it an event? */
 		if (ar[0].revents) {
 			char *event;
-			
+			struct stat trash;
+			int fexists;
+
+			/* check for existence of a lockfile */
+			fexists = (stat(lockfile, &trash) == 0);
+
 			/* this shouldn't happen */
 			if (!ar[0].revents & POLLIN) {
 				acpid_log(LOG_DEBUG,
@@ -202,21 +222,37 @@ main(int argc, char **argv)
 				continue;
 			}
 
-			/* read and handle an event */
+			/* read an event */
 			event = read_line(event_fd);
+
+			/* if we're locked, don't process the event */
+			if (fexists) {
+				if (logevents) {
+					acpid_log(LOG_INFO,
+					    "lockfile present, not processing "
+					    "event \"%s\"\n", event);
+				}
+				continue;
+			}
+
+			/* handle the event */
 			if (event) {
-				acpid_log(LOG_INFO,
-				    "received event \"%s\"\n", event);
+				if (logevents) {
+					acpid_log(LOG_INFO,
+					    "received event \"%s\"\n", event);
+				}
 				acpid_handle_event(event);
-				acpid_log(LOG_INFO,
-				    "completed event \"%s\"\n", event);
+				if (logevents) {
+					acpid_log(LOG_INFO,
+					    "completed event \"%s\"\n", event);
+				}
 			} else if (errno == EPIPE) {
 				acpid_log(LOG_WARNING,
 				    "events file connection closed\n");
 				break;
 			} else {
 				static int nerrs;
-				if (++nerrs >= ACPI_MAX_ERRS) {
+				if (++nerrs >= ACPID_MAX_ERRS) {
 					acpid_log(LOG_ERR,
 					    "too many errors reading "
 					    "events file - aborting\n");
@@ -246,6 +282,7 @@ main(int argc, char **argv)
 				    strerror(errno));
 				continue;
 			}
+			fcntl(cli_fd, F_SETFD, FD_CLOEXEC);
 			snprintf(buf, sizeof(buf)-1, "%d[%d:%d]",
 				creds.pid, creds.uid, creds.gid);
 			acpid_add_client(cli_fd, buf);
@@ -268,10 +305,13 @@ handle_cmdline(int *argc, char ***argv)
 		{"debug", 0, 0, 'd'},
 		{"eventfile", 1, 0, 'e'},
 		{"foreground", 0, 0, 'f'},
+		{"logevents", 0, 0, 'l'},
 		{"socketgroup", 1, 0, 'g'},
 		{"socketmode", 1, 0, 'm'},
 		{"socketfile", 1, 0, 's'},
 		{"nosocket", 1, 0, 'S'},
+		{"pidfile", 1, 0, 'p'},
+		{"lockfile", 1, 0, 'L'},
 		{"version", 0, 0, 'v'},
 		{"help", 0, 0, 'h'},
 		{NULL, 0, 0, 0},
@@ -281,10 +321,13 @@ handle_cmdline(int *argc, char ***argv)
 		"Increase debugging level (implies -f).",/* debug */
 		"Use the specified file for events.",	/* eventfile */
 		"Run in the foreground.",		/* foreground */
+		"Log all event activity.",		/* logevents */
 		"Set the group on the socket file.",	/* socketgroup */
 		"Set the permissions on the socket file.",/* socketmode */
 		"Use the specified socket file.",	/* socketfile */
 		"Do not listen on a UNIX socket (overrides -s).",/* nosocket */
+		"Use the specified PID file.",		/* pidfile */
+		"Use the specified lockfile to stop processing.", /* pidfile */
 		"Print version information.",		/* version */
 		"Print this message.",			/* help */
 	};
@@ -294,7 +337,8 @@ handle_cmdline(int *argc, char ***argv)
 
 	for (;;) {
 		int i;
-		i = getopt_long(*argc, *argv, "c:de:fg:m:s:Svh", opts, NULL);
+		i = getopt_long(*argc, *argv,
+		    "c:de:flg:m:s:Sp:L:vh", opts, NULL);
 		if (i == -1) {
 			break;
 		}
@@ -312,6 +356,9 @@ handle_cmdline(int *argc, char ***argv)
 		case 'f':
 			foreground = 1;
 			break;
+		case 'l':
+			logevents = 1;
+			break;
 		case 'g':
 			socketgroup = optarg;
 			break;
@@ -324,6 +371,12 @@ handle_cmdline(int *argc, char ***argv)
 		case 'S':
 			nosocket = 1;
 			break;
+		case 'p':
+			pidfile = optarg;
+			break;
+		case 'L':
+			lockfile = optarg;
+			break;
 		case 'v':
 			printf(PACKAGE "-" VERSION "\n");
 			exit(EXIT_SUCCESS);
@@ -387,7 +440,11 @@ daemonize(void)
 	umask(0);
 
 	/* get outta the way */
-	chdir("/");
+	if (chdir("/") < 0) {
+		fprintf(stderr, "%s: chdir(\"/\"): %s\n", progname,
+		        strerror(errno));
+		return -1;
+	}
 
 	return 0;
 }
@@ -399,7 +456,7 @@ open_log(void)
 	int log_opts;
 
 	/* open /dev/null */
-	nullfd = open("/dev/null", O_RDONLY, 0640);
+	nullfd = open("/dev/null", O_RDONLY);
 	if (nullfd < 0) {
 		fprintf(stderr, "%s: can't open %s: %s\n", progname, 
 			"/dev/null", strerror(errno));
@@ -431,23 +488,57 @@ open_log(void)
 	return 0;
 }
 
+static int
+create_pidfile(void)
+{
+	int fd;
+
+	/* JIC */
+	unlink(pidfile);
+
+	/* open the pidfile */
+	fd = open(pidfile, O_WRONLY|O_CREAT|O_EXCL, 0644);
+	if (fd >= 0) {
+		FILE *f;
+
+		/* write our pid to it */
+		f = fdopen(fd, "w");
+		if (f != NULL) {
+			fprintf(f, "%d\n", getpid());
+			fclose(f);
+			/* leave the fd open */
+			return 0;
+		}
+		close(fd);
+	}
+
+	/* something went wrong */
+	acpid_log(LOG_ERR, "can't create pidfile %s: %s\n",
+		    pidfile, strerror(errno));
+	return -1;
+}
+
 static void
-clean_exit(int sig)
+clean_exit(int sig __attribute__((unused)))
 {
 	acpid_cleanup_rules(1);
 	acpid_log(LOG_NOTICE, "exiting\n");
+	unlink(pidfile);
 	exit(EXIT_SUCCESS);
 }
 
 static void
-reload_conf(int sig)
+reload_conf(int sig __attribute__((unused)))
 {
 	acpid_log(LOG_NOTICE, "reloading configuration\n");
 	acpid_cleanup_rules(0);
 	acpid_read_conf(confdir);
 }
 
-int 
+int
+#ifdef __GNUC__
+__attribute__((format(printf, 2, 3)))
+#endif
 acpid_log(int level, const char *fmt, ...)
 {
 	va_list args;
