@@ -30,11 +30,13 @@
 #include <sys/stat.h>
 
 #define ACPI_PROCDIR 		"/proc/acpi"
-#define ACPI_EVENTFILE		ACPI_PROCDIR "/event"
-#define ACPI_CONFDIR		"/etc/acpi/events"
-#define ACPI_SOCKETFILE		"/var/run/acpid.socket"
-#define ACPI_SOCKETMODE		0666
-#define ACPI_MAX_ERRS		5
+#define ACPID_EVENTFILE		ACPI_PROCDIR "/event"
+#define ACPID_CONFDIR		"/etc/acpi/events"
+#define ACPID_SOCKETFILE	"/var/run/acpid.socket"
+#define ACPID_SOCKETMODE	0666
+#define ACPID_PIDFILE		"/var/run/acpid.pid"
+#define ACPID_LOCKFILE		"/var/lock/acpid"
+#define ACPID_MAX_ERRS		5
 
 #define PACKAGE 		"acpid"
 
@@ -42,6 +44,7 @@
  * acpid.c
  */
 extern int acpid_debug;
+extern int logevents;
 extern int acpid_log(int level, const char *fmt, ...);
 
 /*
